clc
clear

% Choose integer MULTIPLIER to increase a problem dimention
    MULTIPLIER=1;                               % 5 or 10 or 40 or ...

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Generate data for the problem
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    n = 10*MULTIPLIER;                          % the number of atoms in fixed distribution
    x = (1:n)' ./ (n+1) - 1/2;                  % values of fixed atoms
    n_y = ceil(10 * n);                         % the number of atoms in approximated distribution
    y = (1:(n_y))' / (n_y+1) - 1/2;             % values of atoms with variable probability
  % parameters for generation probabilities of fixed distribution
    q1 = 0.3;
    q2 = 0.6;
    q3 = 0.8;
    lambda1 = 0.6;
    lambda2 = 0.0;
    lambda3 = 0.4;
    prob = zeros(size(x));
  % probabilities generation for fixed distribution  
    n1=10; 
    for k = 1:n; k1=mod(k,n1);
        prob(k) = prob(k) + lambda1 * nchoosek(n1, k1) * q1 ^ k1 * (1 - q1) ^ (n1 - k1);
        prob(k) = prob(k) + lambda2 * nchoosek(n1, k1) * q2 ^ k1 * (1 - q2) ^ (n1 - k1);
        prob(k) = prob(k) + lambda3 * nchoosek(n1, k1) * q3 ^ k1 * (1 - q3) ^ (n1 - k1);
    end
    prob = prob / sum(prob);

   
% Generate parameters for a set of SSD Cvar constraints
MAX_CONSTR = 100*MULTIPLIER;

if n <= MAX_CONSTR
    m1 = n-1;
    alpha1 = zeros(m1, 1);
    A1 = zeros(m1, 1);
    for k = 1:m1
        alpha1(k) = sum(prob(x <= x(k)));
        A1(k) = CVaR(x, prob, alpha1(k));
    end
else
    m1 = MAX_CONSTR;
    alpha1 = zeros(m1, 1);
    A1 = zeros(m1, 1);
    for k = 1:m1
        alpha1(k) = (k - 1/2)/m1;
        A1(k) = CVaR(x, prob, alpha1(k));
    end
end

% Epsilon for Moment constraint
epsilon_var = 0.003;

% Set variables names - variables are probabilities of atoms in approximating distribution
varibles_p={}; for i=1:n_y; varibles_p{i}=['p' int2str(i)]; end

% Set bounds for constraints
EX = prob'*x;
sigma_sq = prob'*x.^ 2+ epsilon_var;

%{
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Create and Solve optimization problem programmatically
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Problem Statement:
problem_statement = sprintf('%s\n',...
'problem_approximation_by_parameter_numyy, maximize',...
'  -entropyr(matrix_p)',...
'Constraint: EP, == 1',...
'  linear(matrix_p)',...
'Constraint: EX, == parameter_ex',...                    
'  linear(matrix_y)',...
'Constraint: sigma_sq, <= parameter_sigma',...       
'  linear(matrix_y2)',...
'MultiConstraint: >= vector_lb',...
'  pCvar(vector_cl,matrix_y)',...
'Box: >=0',...
'Solver: init_point = 0.01');


% Pack data
count=1;        toolboxstruc_arr(count) = tbpsg_matrix_pack('matrix_p',ones(1,n_y),varibles_p);
count=count+1;  toolboxstruc_arr(count) = tbpsg_matrix_pack('matrix_y',y',varibles_p);
count=count+1;  toolboxstruc_arr(count) = tbpsg_matrix_pack('matrix_y2', y'.^2,varibles_p);
count=count+1;  toolboxstruc_arr(count) = tbpsg_vector_pack('vector_lb',A1);
count=count+1;  toolboxstruc_arr(count) = tbpsg_vector_pack('vector_cl',alpha1);

count=count+1;  toolboxstruc_arr(count) = tbpsg_parameter_pack('numy',n_y);
count=count+1;  toolboxstruc_arr(count) = tbpsg_parameter_pack('ex',EX);
count=count+1;  toolboxstruc_arr(count) = tbpsg_parameter_pack('sigma',sigma_sq);

%{
% Open optimization problem in Toolbox GUI
tbpsg_toolbox(problem_statement,toolboxstruc_arr);
%}

% Solve optimization problem
[solution_str, outargstruc_arr] =  tbpsg_run(problem_statement,toolboxstruc_arr);
%}

%{
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Create user subroutine
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Note: uncomment previously section with creation of the problem statement and structure with data

tbpsg_create_user_subroutine(problem_statement,toolboxstruc_arr,'.\','entropy_maximization_subroutine');
%}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Solve optimization problem with user subroutine
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[solution_str,outargstruc_arr] = entropy_maximization_subroutine(ones(1,n_y),varibles_p,y',y'.^2,A1,alpha1,n_y,EX,sigma_sq);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Extract optimization results
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Display optimal probabilities of approximating distribution
p=tbpsg_optimal_point_data(solution_str, outargstruc_arr)';
format long
disp('p'); disp(p);

% Output Dual constraints
dual_data = tbpsg_dual_data(solution_str, outargstruc_arr);
disp(' ')
disp('Dual');disp(dual_data)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Available PSG functions for viewing solution outputs
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Extract structure containing PSG solution reports:
%output_structure = tbpsg_solution_struct(solution_str, outargstruc_arr);

%output = tbpsg_isoptimal(solution_str, outargstruc_arr);

%output = tbpsg_function_data(solution_str, outargstruc_arr);

%output = tbpsg_function_names(solution_str, outargstruc_arr);

%output = tbpsg_time(solution_str, outargstruc_arr);

%output = tbpsg_optimal_point_vars(solution_str, outargstruc_arr);

%output = tbpsg_constraints_vars(solution_str, outargstruc_arr);

%output = tbpsg_slack_data(solution_str, outargstruc_arr);

%output = tbpsg_dual_data(solution_str, outargstruc_arr);

%output = tbpsg_vector_constraint_data(solution_str, outargstruc_arr);

%output = tbpsg_vector_dual_data(solution_str, outargstruc_arr);

%output = tbpsg_vector_slack_data(solution_str, outargstruc_arr);

%output = tbpsg_matrix_data(solution_str, outargstruc_arr);

%output = tbpsg_matrix_vars(solution_str, outargstruc_arr);

%output = tbpsg_vector_data(solution_str, outargstruc_arr);




